function y = mttflipdim(x, dim)
%MTTFLIPDIM Flip array along specified dimension.
%
%   MTTFLIPDIM(X,DIM) returns X with dimension DIM flipped.
%   For example, MTTFLIPDIM(X,1) where
%
%       X = 1 4  produces  3 6
%           2 5            2 5
%           3 6            1 4
%
%   MTTFLIPDIM(X) flips X along the first non-singleton dimension.  As a
%   consequence, MTTFLIPDIM(X), where X is a vector, will reverse the order
%   of the elements of X.
%
%   See also MTTFLIPLR, MTTFLIPUD, ROT90, PERMUTE.

%   This program is a part of the MTT (MATLAB Tips & Tricks) Toolbox.
%   Copyright (c) 2002 Peter J. Acklam. All rights reserved.
%
%   This program is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.  See the file COPYING for details.
%
%   Author:      Peter J. Acklam
%   Time-stamp:  2003-03-31 18:32:21 +0200
%   E-mail:      pjacklam@online.no
%   URL:         http://home.online.no/~pjacklam

   % check number of input arguments
   error(nargchk(1, 2, nargin));

   % if no dimension argument, use first non-singleton dimension
   if nargin < 2
      k = find(size(x) ~= 1);   % find non-singleton dimensions
      if isempty(k)             % no non-singleton dimensions
         y = x;
         return;
      else                      % get first non-singleton dimension
         dim = k(1);
      end
   else
      dim = floor(dim);
      if dim < 1
         error('DIM must be a positive integer.');
      end
   end

   % now do the actual flipping
   n = size(x, dim);
   if n <= 1                            % if length <= 1
      y = x;                            %   flipping changes nothing
   else                                 % or else
      idx{ndims(x)} = ':';              %   initialize subscript array
      idx(:)        = idx(end);         %   fill with colons
      idx{dim}      = n : -1 : 1;       %   reverse appropriate dimension
      y             = x(idx{:});        %
   end
