function y = mttflipdims(x, dims)
%MTTFLIPDIMS Flip array along specified dimensions.
%
%   MTTFLIPDIMS(X, DIMS) flips the array X along all dimensions specified in
%   DIMS.  DIMS must be a vector of real values >= 1.
%
%   If DIMS is not specified, MTTFLIPDIMS flips along the first non-
%   singleton dimension.  As a consequence, MTTFLIPDIMS(X), where X is a
%   vector, will reverse the order of the elements of X.
%
%   Note that the multiplicity of the elements in DIMS does matter.
%   Flipping an even number of times along a dimension changes nothing.
%
%   See also MTTFLIPDIM, MTTFLIPLR, MTTFLIPUD, MTTFLIPFB, ROT90, PERMUTE.

%   This program is a part of the MTT (MATLAB Tips & Tricks) Toolbox.
%   Copyright (c) 2002 Peter J. Acklam. All rights reserved.
%
%   This program is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.  See the file COPYING for details.
%
%   Author:      Peter J. Acklam
%   Time-stamp:  2003-03-31 18:32:14 +0200
%   E-mail:      pjacklam@online.no
%   URL:         http://home.online.no/~pjacklam

   % check number of input arguments
   nargsin = nargin;
   error(nargchk(1, 2, nargin));

   sx = size(x);
   dx = ndims(x);

   % if dimension vector is not given, get first non-singleton dimension
   if nargsin < 2                       % no dimension vector specified
      k = find(sx > 1);                 % find non-singleton dimensions
      if isempty(k)                     % if no non-singleton dimensions
         y = x;                         %   output is identical to input
         return;                        %   bail out
      end
      dims = k(1);                      % pick first non-singleton dimension

   % if dimension vector is given, check it
   else
      if sum(size(dims) > 1) > 1 | any(dims < 1)
         error('DIMS must be a vector of values >= 1.');
      end
      dims = fix(dims);                 % make sure values are integers
      dims = dims(dims <= dx);          % dim is singleton if dim > ndims(x)
      dims = dims(sx(dims) > 1);        % dim is singleton if sx(dim) = 1
      if isempty(dims)                  % if no non-singleton dimensions
         y = x;                         %   output is identical to input
         return;                        %   bail out
      end

   end

   % create the list of subscripts
   idx{dx} = ':';
   idx(:) = idx(end);
   for i = 1:length(dims)
      n = sx(dims(i));                  % length along dimension dims(i)
      if n > 1                          % flipping is a null-op if n <= 1
         if isequal(idx{dims(i)}, ':')  % if dimension not flipped
            idx{dims(i)} = n:-1:1;      %    then flip it
         else                           % otherwise
            idx{dims(i)} = ':';         %    flip again (i.e., no change)
         end
      end
   end

   % now flip dimensions
   y = x(idx{:});
